use crate::prelude::*;
use log::error;
use tokio::task::spawn_blocking;

/// Returns true if the provided master password is valid
#[inline]
async fn check_master_pass(
    bot: &Throttle<Bot>,
    msg: &Message,
    db: &DatabaseConnection,
    master_pass: &str,
) -> crate::Result<Option<Message>> {
    let user_id = msg.from().ok_or(NoUserInfo)?.id.0;
    let model = MasterPass::get(user_id, db).await?;

    let is_valid = match model {
        Some(model) => {
            let master_pass = master_pass.to_owned();
            spawn_blocking(move || model.verify(&master_pass)).await?
        }
        None => {
            error!("User was put into the GetMasterPass state with no master password set");
            let msg = bot
                .send_message(
                    msg.chat.id,
                    "No master password set. Use /cancel and set it by using /set_master_pass",
                )
                .await?;
            return Ok(Some(msg));
        }
    };

    if !is_valid {
        let msg = bot
            .send_message(msg.chat.id, "Wrong master password. Try again")
            .await?;
        return Ok(Some(msg));
    }
    Ok(None)
}

pub async fn get_master_pass(
    bot: Throttle<Bot>,
    msg: Message,
    db: DatabaseConnection,
    dialogue: MainDialogue,
    next: PackagedHandler<String>,
) -> crate::Result<()> {
    super::generic::generic(
        bot,
        msg,
        db,
        dialogue,
        |bot, msg, db, text| Box::pin(check_master_pass(bot, msg, db, text)),
        "Couldn't get the text of the message. Send the master password again",
        next,
    )
    .await
}
